@@ -1,3 +1,4 @@
+//go:build !js
 // +build !js
 
 package stun
@@ -16,6 +17,17 @@ import (
 	"time"
 )
 
+var (
+	errClientAlreadyStopped = errors.New("already stopped")
+	errClientReadTimedOut   = errors.New("read timed out")
+	errClientWriteTimedOut  = errors.New("write timed out")
+	errClientCloseError     = errors.New("close error")
+	errClientSetHandler     = errors.New("set handler error")
+	errClientStart          = errors.New("start error")
+	errClientAgentCantStop  = errors.New("agent dont want to stop")
+	errClientStartRefused   = errors.New("start refused")
+)
+
 type TestAgent struct {
 	h Handler
 	e chan Event
@@ -72,7 +84,12 @@ func BenchmarkClient_Do(b *testing.B) {
 	if err != nil {
 		log.Fatal(err)
 	}
-	defer client.Close()
+	defer func() {
+		if closeErr := client.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
+
 	noopF := func(event Event) {
 		// pass
 	}
@@ -83,7 +100,7 @@ func BenchmarkClient_Do(b *testing.B) {
 			}
 		}()
 		m := New()
-		m.NewTransactionID()
+		m.NewTransactionID() // nolint:errcheck,gosec
 		m.Encode()
 		for pb.Next() {
 			if err := client.Do(m, noopF); err != nil {
@@ -113,7 +130,7 @@ func (t *testConnection) Close() error {
 	t.stoppedMux.Lock()
 	defer t.stoppedMux.Unlock()
 	if t.stopped {
-		return errors.New("already stopped")
+		return errClientAlreadyStopped
 	}
 	t.stopped = true
 	return nil
@@ -136,8 +153,8 @@ func TestClosedOrPanic(t *testing.T) {
 	closedOrPanic(ErrAgentClosed)
 	func() {
 		defer func() {
-			r := recover()
-			if r != io.EOF {
+			r, ok := recover().(error)
+			if !ok || !errors.Is(r, io.EOF) {
 				t.Error(r)
 			}
 		}()
@@ -160,7 +177,7 @@ func TestClient_Start(t *testing.T) {
 				copy(i, response.Raw)
 				return len(response.Raw), nil
 			case <-time.After(time.Millisecond * 10):
-				return 0, errors.New("read timed out")
+				return 0, errClientReadTimedOut
 			}
 		},
 		write: func(bytes []byte) (int, error) {
@@ -170,7 +187,7 @@ func TestClient_Start(t *testing.T) {
 				t.Log("writing")
 				return len(bytes), nil
 			case <-time.After(time.Millisecond * 10):
-				return 0, errors.New("write timed out")
+				return 0, errClientWriteTimedOut
 			}
 		},
 	}
@@ -206,7 +223,7 @@ func TestClient_Start(t *testing.T) {
 	t.Log("starting the second transaction")
 	if err := c.Start(m, func(e Event) {
 		t.Error("should not be called")
-	}); err != ErrTransactionExists {
+	}); !errors.Is(err, ErrTransactionExists) {
 		t.Errorf("unexpected error %v", err)
 	}
 	read <- struct{}{}
@@ -353,7 +370,7 @@ func TestClientAgentError(t *testing.T) {
 		if event.Error == nil {
 			t.Error("error expected")
 		}
-	}); err != io.ErrUnexpectedEOF {
+	}); !errors.Is(err, io.ErrUnexpectedEOF) {
 		t.Error("error expected")
 	}
 }
@@ -377,7 +394,7 @@ func TestClientConnErr(t *testing.T) {
 	if err := c.Do(m, nil); err == nil {
 		t.Error("error expected")
 	}
-	if err := c.Do(m, NoopHandler); err == nil {
+	if err := c.Do(m, NoopHandler()); err == nil {
 		t.Error("error expected")
 	}
 }
@@ -402,7 +419,7 @@ func TestClientConnErrStopErr(t *testing.T) {
 		}
 	}()
 	m := MustBuild(TransactionID)
-	if err := c.Do(m, NoopHandler); err == nil {
+	if err := c.Do(m, NoopHandler()); err == nil {
 		t.Error("error expected")
 	}
 }
@@ -434,7 +451,7 @@ func TestNewClientNoConnection(t *testing.T) {
 	if c != nil {
 		t.Error("c should be nil")
 	}
-	if err != ErrNoConnection {
+	if !errors.Is(err, ErrNoConnection) {
 		t.Error("bad error")
 	}
 }
@@ -457,6 +474,7 @@ func TestDialError(t *testing.T) {
 		t.Fatal("error expected")
 	}
 }
+
 func TestClientCloseErr(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
@@ -475,7 +493,7 @@ func TestClientCloseErr(t *testing.T) {
 		log.Fatal(err)
 	}
 	defer func() {
-		if err, ok := c.Close().(CloseErr); !ok || err.AgentErr != io.ErrUnexpectedEOF {
+		if err, ok := c.Close().(CloseErr); !ok || !errors.Is(err.AgentErr, io.ErrUnexpectedEOF) { //nolint:errorlint
 			t.Error("unexpected close err")
 		}
 	}()
@@ -484,7 +502,7 @@ func TestClientCloseErr(t *testing.T) {
 func TestWithNoConnClose(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
-	closeErr := errors.New("close error")
+	closeErr := errClientCloseError
 	conn := &testConnection{
 		b: response.Raw,
 		close: func() error {
@@ -495,7 +513,7 @@ func TestWithNoConnClose(t *testing.T) {
 		WithAgent(errorAgent{
 			closeErr: nil,
 		}),
-		WithNoConnClose,
+		WithNoConnClose(),
 	)
 	if err != nil {
 		log.Fatal(err)
@@ -567,10 +585,10 @@ func TestClientGC(t *testing.T) {
 }
 
 func TestClientCheckInit(t *testing.T) {
-	if err := (&Client{}).Indicate(nil); err != ErrClientNotInitialized {
+	if err := (&Client{}).Indicate(nil); !errors.Is(err, ErrClientNotInitialized) {
 		t.Error("unexpected error")
 	}
-	if err := (&Client{}).Do(nil, nil); err != ErrClientNotInitialized {
+	if err := (&Client{}).Do(nil, nil); !errors.Is(err, ErrClientNotInitialized) {
 		t.Error("unexpected error")
 	}
 }
@@ -598,7 +616,7 @@ func TestClientFinalizer(t *testing.T) {
 	}
 	c, err := NewClient(conn)
 	if err != nil {
-		log.Fatal(err)
+		log.Panic(err)
 	}
 	clientFinalizer(c)
 	clientFinalizer(c)
@@ -616,12 +634,12 @@ func TestClientFinalizer(t *testing.T) {
 		}),
 	)
 	if err != nil {
-		log.Fatal(err)
+		log.Panic(err)
 	}
 	clientFinalizer(c)
 	reader := bufio.NewScanner(buf)
 	var lines int
-	var expectedLines = []string{
+	expectedLines := []string{
 		"client: called finalizer on non-closed client: client not initialized",
 		"client: called finalizer on non-closed client",
 		"client: called finalizer on non-closed client: failed to close: " +
@@ -642,7 +660,7 @@ func TestClientFinalizer(t *testing.T) {
 }
 
 func TestCallbackWaitHandler(t *testing.T) {
-	h := callbackWaitHandlerPool.Get().(*callbackWaitHandler)
+	h := callbackWaitHandlerPool.Get().(*callbackWaitHandler) //nolint:forcetypeassert
 	for i := 0; i < 100; i++ {
 		h.setCallback(func(event Event) {})
 		go func() {
@@ -729,7 +747,11 @@ func TestClientRetransmission(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := &manualClock{current: time.Now()}
 	agent := &manualAgent{}
@@ -792,7 +814,11 @@ func testClientDoConcurrent(t *testing.T, concurrency int) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := &manualClock{current: time.Now()}
 	agent := &manualAgent{}
@@ -822,7 +848,7 @@ func testClientDoConcurrent(t *testing.T, concurrency int) {
 			for {
 				readN, readErr := connL.Read(buf)
 				if readErr != nil {
-					if readErr == io.EOF {
+					if errors.Is(readErr, io.EOF) {
 						break
 					}
 					t.Error(readErr)
@@ -852,10 +878,10 @@ func testClientDoConcurrent(t *testing.T, concurrency int) {
 }
 
 func TestClient_DoConcurrent(t *testing.T) {
-	t.Parallel()
 	for _, concurrency := range []int{
 		1, 5, 10, 25, 100, 500,
 	} {
+		concurrency := concurrency
 		t.Run(fmt.Sprintf("%d", concurrency), func(t *testing.T) {
 			testClientDoConcurrent(t, concurrency)
 		})
@@ -875,23 +901,23 @@ func (c errorCollector) Close() error { return c.closeError }
 
 func TestNewClient(t *testing.T) {
 	t.Run("SetCallbackError", func(t *testing.T) {
-		setHandlerError := errors.New("set handler error")
+		setHandlerError := errClientSetHandler
 		if _, createErr := NewClient(noopConnection{},
 			WithAgent(&errorAgent{
 				setHandlerError: setHandlerError,
 			}),
-		); createErr != setHandlerError {
+		); !errors.Is(createErr, setHandlerError) {
 			t.Errorf("unexpected error returned: %v", createErr)
 		}
 	})
 	t.Run("CollectorStartError", func(t *testing.T) {
-		startError := errors.New("start error")
+		startError := errClientStart
 		if _, createErr := NewClient(noopConnection{},
 			WithAgent(&TestAgent{}),
 			WithCollector(errorCollector{
 				startError: startError,
 			}),
-		); createErr != startError {
+		); !errors.Is(createErr, startError) {
 			t.Errorf("unexpected error returned: %v", createErr)
 		}
 	})
@@ -899,7 +925,7 @@ func TestNewClient(t *testing.T) {
 
 func TestClient_Close(t *testing.T) {
 	t.Run("CollectorCloseError", func(t *testing.T) {
-		closeErr := errors.New("start error")
+		closeErr := errClientStart
 		c, createErr := NewClient(noopConnection{},
 			WithCollector(errorCollector{
 				closeError: closeErr,
@@ -910,7 +936,7 @@ func TestClient_Close(t *testing.T) {
 			t.Errorf("unexpected create error returned: %v", createErr)
 		}
 		gotCloseErr := c.Close()
-		if gotCloseErr != closeErr {
+		if !errors.Is(gotCloseErr, closeErr) {
 			t.Errorf("unexpected close error returned: %v", gotCloseErr)
 		}
 	})
@@ -970,7 +996,7 @@ func TestClientClosedStart(t *testing.T) {
 	if closeErr := c.Close(); closeErr != nil {
 		t.Error(closeErr)
 	}
-	if startErr := c.start(&clientTransaction{}); startErr != ErrClientClosed {
+	if startErr := c.start(&clientTransaction{}); !errors.Is(startErr, ErrClientClosed) {
 		t.Error("should error")
 	}
 }
@@ -979,7 +1005,11 @@ func TestWithNoRetransmit(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := &manualClock{current: time.Now()}
 	agent := &manualAgent{}
@@ -1023,7 +1053,7 @@ func TestWithNoRetransmit(t *testing.T) {
 		gotReads <- struct{}{}
 	}()
 	if doErr := c.Do(MustBuild(response, BindingRequest), func(event Event) {
-		if event.Error != ErrTransactionTimeOut {
+		if !errors.Is(event.Error, ErrTransactionTimeOut) {
 			t.Error("unexpected error")
 		}
 	}); doErr != nil {
@@ -1042,7 +1072,11 @@ func TestClientRTOStartErr(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	shouldWait := false
 	shouldWaitMux := new(sync.RWMutex)
@@ -1134,7 +1168,7 @@ func TestClientRTOStartErr(t *testing.T) {
 	done := make(chan struct{})
 	go func() {
 		if doErr := c.Do(MustBuild(response, BindingRequest), func(event Event) {
-			if event.Error != ErrClientClosed {
+			if !errors.Is(event.Error, ErrClientClosed) {
 				t.Error(event.Error)
 			}
 		}); doErr != nil {
@@ -1154,7 +1188,11 @@ func TestClientRTOWriteErr(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	shouldWait := false
 	shouldWaitMux := new(sync.RWMutex)
@@ -1184,7 +1222,7 @@ func TestClientRTOWriteErr(t *testing.T) {
 		c              *Client
 		startClientErr error
 	)
-	agentStopErr := errors.New("agent dont want to stop")
+	agentStopErr := errClientAgentCantStop
 	agent.stop = func(id [TransactionIDSize]byte) error {
 		return agentStopErr
 	}
@@ -1209,7 +1247,9 @@ func TestClientRTOWriteErr(t *testing.T) {
 				t.Log("clock locked")
 				<-clockLocked
 				t.Log("closing connection")
-				connL.Close()
+				if closeErr := connL.Close(); closeErr != nil {
+					panic(closeErr)
+				}
 				t.Log("connection closed, unlocking clock")
 				clockWait <- struct{}{}
 				t.Log("clock unlocked")
@@ -1248,13 +1288,14 @@ func TestClientRTOWriteErr(t *testing.T) {
 	done := make(chan struct{})
 	go func() {
 		if doErr := c.Do(MustBuild(response, BindingRequest), func(event Event) {
-			if e, ok := event.Error.(StopErr); !ok {
+			var e StopErr
+			if !errors.As(event.Error, &e) {
 				t.Error(event.Error)
 			} else {
-				if e.Err != agentStopErr {
+				if !errors.Is(e.Err, agentStopErr) {
 					t.Error("incorrect agent error")
 				}
-				if e.Cause != io.ErrClosedPipe {
+				if !errors.Is(e.Cause, io.ErrClosedPipe) {
 					t.Error("incorrect connection error")
 				}
 			}
@@ -1275,7 +1316,11 @@ func TestClientRTOAgentErr(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := callbackClock(time.Now)
 	agent := &manualAgent{}
@@ -1285,7 +1330,7 @@ func TestClientRTOAgentErr(t *testing.T) {
 		c              *Client
 		startClientErr error
 	)
-	agentStartErr := errors.New("start refused")
+	agentStartErr := errClientStartRefused
 	agent.start = func(id [TransactionIDSize]byte, deadline time.Time) error {
 		t.Log("start", attempt)
 		if attempt == 0 {
@@ -1321,7 +1366,7 @@ func TestClientRTOAgentErr(t *testing.T) {
 	}()
 	t.Log("starting")
 	if doErr := c.Do(MustBuild(response, BindingRequest), func(event Event) {
-		if event.Error != agentStartErr {
+		if !errors.Is(event.Error, agentStartErr) {
 			t.Error(event.Error)
 		}
 	}); doErr != nil {
@@ -1339,7 +1384,11 @@ func TestClient_HandleProcessError(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	response.Encode()
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := callbackClock(time.Now)
 	agent := &manualAgent{}
@@ -1380,7 +1429,11 @@ func TestClient_HandleProcessError(t *testing.T) {
 func TestClientImmediateTimeout(t *testing.T) {
 	response := MustBuild(TransactionID, BindingSuccess)
 	connL, connR := net.Pipe()
-	defer connL.Close()
+	defer func() {
+		if closeErr := connL.Close(); closeErr != nil {
+			panic(closeErr)
+		}
+	}()
 	collector := new(manualCollector)
 	clock := &manualClock{current: time.Now()}
 	rto := time.Second * 1
@@ -1426,8 +1479,8 @@ func TestClientImmediateTimeout(t *testing.T) {
 		}
 		gotReads <- struct{}{}
 	}()
-	c.Start(MustBuild(response, BindingRequest), func(e Event) {
-		if e.Error == ErrTransactionTimeOut {
+	c.Start(MustBuild(response, BindingRequest), func(e Event) { // nolint:errcheck,gosec
+		if errors.Is(e.Error, ErrTransactionTimeOut) {
 			t.Error("unexpected error")
 		}
 	})
