@@ -1,3 +1,4 @@
+// Package main implements a CLI tool which acts as a STUN client
 package main
 
 import (
@@ -19,13 +20,14 @@ func main() {
 	if addr == "" {
 		addr = "stun.l.google.com:19302"
 	}
-	c, err := stun.Dial("udp", addr)
+	// we only try the first address, so restrict ourselves to IPv4
+	c, err := stun.Dial("udp4", addr)
 	if err != nil {
 		log.Fatal("dial:", err)
 	}
 	if err = c.Do(stun.MustBuild(stun.TransactionID, stun.BindingRequest), func(res stun.Event) {
 		if res.Error != nil {
-			log.Fatalln(err)
+			log.Fatalln(res.Error)
 		}
 		var xorAddr stun.XORMappedAddress
 		if getErr := xorAddr.GetFrom(res.Message); getErr != nil {
