@@ -1,3 +1,4 @@
+//go:build !js
 // +build !js
 
 package stun
@@ -7,6 +8,7 @@ import (
 	"encoding/base64"
 	"encoding/binary"
 	"encoding/csv"
+	"errors"
 	"fmt"
 	"hash/crc64"
 	"io"
@@ -69,7 +71,7 @@ func BenchmarkMessage_Write(b *testing.B) {
 }
 
 func TestMessageType_Value(t *testing.T) {
-	var tests = []struct {
+	tests := []struct {
 		in  MessageType
 		out uint16
 	}{
@@ -87,7 +89,7 @@ func TestMessageType_Value(t *testing.T) {
 }
 
 func TestMessageType_ReadValue(t *testing.T) {
-	var tests = []struct {
+	tests := []struct {
 		in  uint16
 		out MessageType
 	}{
@@ -105,7 +107,7 @@ func TestMessageType_ReadValue(t *testing.T) {
 }
 
 func TestMessageType_ReadWriteValue(t *testing.T) {
-	var tests = []MessageType{
+	tests := []MessageType{
 		{Method: MethodBinding, Class: ClassRequest},
 		{Method: MethodBinding, Class: ClassSuccessResponse},
 		{Method: MethodBinding, Class: ClassErrorResponse},
@@ -190,20 +192,21 @@ func TestMessage_AttrLengthLessThanHeader(t *testing.T) {
 	mDecoded := New()
 	binary.BigEndian.PutUint16(m.Raw[2:4], 2) // rewrite to bad length
 	_, err := mDecoded.ReadFrom(bytes.NewReader(m.Raw[:20+2]))
-	switch e := err.(type) {
-	case *DecodeErr:
+	var e *DecodeErr
+	if errors.As(err, &e) {
 		if !e.IsPlace(DecodeErrPlace{"attribute", "header"}) {
 			t.Error(e, "bad place")
 		}
-	default:
+	} else {
 		t.Error(err, "should be bad format")
 	}
 }
 
 func TestMessage_AttrSizeLessThanLength(t *testing.T) {
 	mType := MessageType{Method: MethodBinding, Class: ClassRequest}
-	messageAttribute := RawAttribute{Length: 4,
-		Value: []byte{1, 2, 3, 4}, Type: 0x1,
+	messageAttribute := RawAttribute{
+		Length: 4,
+		Value:  []byte{1, 2, 3, 4}, Type: 0x1,
 	}
 	messageAttributes := Attributes{
 		messageAttribute,
@@ -218,12 +221,12 @@ func TestMessage_AttrSizeLessThanLength(t *testing.T) {
 	bin.PutUint16(m.Raw[2:4], 5) // rewrite to bad length
 	mDecoded := New()
 	_, err := mDecoded.ReadFrom(bytes.NewReader(m.Raw[:20+5]))
-	switch e := err.(type) {
-	case *DecodeErr:
+	var e *DecodeErr
+	if errors.As(err, &e) {
 		if !e.IsPlace(DecodeErrPlace{"attribute", "value"}) {
 			t.Error(e, "bad place")
 		}
-	default:
+	} else {
 		t.Error(err, "should be bad format")
 	}
 }
@@ -237,7 +240,7 @@ func (r unexpectedEOFReader) Read(b []byte) (int, error) {
 func TestMessage_ReadFromError(t *testing.T) {
 	mDecoded := New()
 	_, err := mDecoded.ReadFrom(unexpectedEOFReader{})
-	if err != io.ErrUnexpectedEOF {
+	if !errors.Is(err, io.ErrUnexpectedEOF) {
 		t.Error(err, "should be", io.ErrUnexpectedEOF)
 	}
 }
@@ -263,7 +266,7 @@ func BenchmarkMessage_WriteTo(b *testing.B) {
 	buf := new(bytes.Buffer)
 	b.ReportAllocs()
 	for i := 0; i < b.N; i++ {
-		m.WriteTo(buf)
+		m.WriteTo(buf) // nolint:errcheck,gosec
 		buf.Reset()
 	}
 }
@@ -499,10 +502,10 @@ func TestMessage_String(t *testing.T) {
 
 func TestIsMessage(t *testing.T) {
 	m := New()
-	NewSoftware("software").AddTo(m)
+	NewSoftware("software").AddTo(m) // nolint:errcheck,gosec
 	m.WriteHeader()
 
-	var tt = [...]struct {
+	tt := [...]struct {
 		in  []byte
 		out bool
 	}{
@@ -511,9 +514,11 @@ func TestIsMessage(t *testing.T) {
 		{[]byte{1, 2, 4}, false},                    // 2
 		{[]byte{1, 2, 4, 5, 6, 7, 8, 9, 20}, false}, // 3
 		{m.Raw, true},                               // 5
-		{[]byte{0, 0, 0, 0, 33, 18,
+		{[]byte{
+			0, 0, 0, 0, 33, 18,
 			164, 66, 0, 0, 0, 0, 0,
-			0, 0, 0, 0, 0, 0, 0, 0}, true}, // 6
+			0, 0, 0, 0, 0, 0, 0, 0,
+		}, true}, // 6
 	}
 	for i, v := range tt {
 		if got := IsMessage(v.in); got != v.out {
@@ -526,7 +531,7 @@ func BenchmarkIsMessage(b *testing.B) {
 	m := New()
 	m.Type = MessageType{Method: MethodBinding, Class: ClassRequest}
 	m.TransactionID = NewTransactionID()
-	NewSoftware("cydev/stun test").AddTo(m)
+	NewSoftware("cydev/stun test").AddTo(m) // nolint:errcheck,gosec
 	m.WriteHeader()
 
 	b.SetBytes(int64(messageHeaderSize))
@@ -541,7 +546,7 @@ func BenchmarkIsMessage(b *testing.B) {
 
 func loadData(tb testing.TB, name string) []byte {
 	name = filepath.Join("testdata", name)
-	f, err := os.Open(name)
+	f, err := os.Open(name) //nolint: gosec
 	if err != nil {
 		tb.Fatal(err)
 	}
@@ -578,7 +583,7 @@ func TestMessageFromBrowsers(t *testing.T) {
 	m := New()
 	for {
 		line, err := reader.Read()
-		if err == io.EOF {
+		if errors.Is(err, io.EOF) {
 			break
 		}
 		if err != nil {
@@ -621,11 +626,15 @@ func BenchmarkMessageFull(b *testing.B) {
 		IP: net.IPv4(213, 1, 223, 5),
 	}
 	for i := 0; i < b.N; i++ {
-		addAttr(b, m, addr)
-		addAttr(b, m, &s)
+		if err := addr.AddTo(m); err != nil {
+			b.Fatal(err)
+		}
+		if err := s.AddTo(m); err != nil {
+			b.Fatal(err)
+		}
 		m.WriteAttributes()
 		m.WriteHeader()
-		Fingerprint.AddTo(m)
+		Fingerprint.AddTo(m) // nolint:errcheck,gosec
 		m.WriteHeader()
 		m.Reset()
 	}
@@ -679,7 +688,7 @@ func TestMessage_Contains(t *testing.T) {
 func ExampleMessage() {
 	buf := new(bytes.Buffer)
 	m := new(Message)
-	m.Build(BindingRequest,
+	m.Build(BindingRequest, // nolint:errcheck,gosec
 		NewTransactionIDSetter([TransactionIDSize]byte{
 			1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1,
 		}),
@@ -705,11 +714,11 @@ func ExampleMessage() {
 	decoded.Raw = make([]byte, 0, 1024) // for ReadFrom that reuses m.Raw
 	// ReadFrom does not allocate internal buffer for reading from io.Reader,
 	// instead it uses m.Raw, expanding it length to capacity.
-	decoded.ReadFrom(buf)
+	decoded.ReadFrom(buf) // nolint:errcheck,gosec
 	fmt.Println("has software:", decoded.Contains(AttrSoftware))
 	fmt.Println("has nonce:", decoded.Contains(AttrNonce))
 	var software Software
-	decoded.Parse(&software) // or software.GetFrom(decoded)
+	decoded.Parse(&software) // nolint:errcheck,gosec
 	// Rule for Parse method is same as for Build.
 	fmt.Println("software:", software)
 	if err := Fingerprint.Check(decoded); err == nil {
@@ -733,7 +742,7 @@ func ExampleMessage() {
 	}
 
 	// Output:
-	// Binding request l=48 attrs=3 id=AQIDBAUGBwgJAAEA buff length: 68
+	// Binding request l=48 attrs=3 id=AQIDBAUGBwgJAAEA, attr0=SOFTWARE attr1=MESSAGE-INTEGRITY attr2=FINGERPRINT  buff length: 68
 	// wrote 68 err <nil>
 	// has software: true
 	// has nonce: false
@@ -765,6 +774,8 @@ func TestAllocations(t *testing.T) {
 	}
 	m := New()
 	for i, s := range setters {
+		s := s
+		i := i
 		allocs := testing.AllocsPerRun(10, func() {
 			m.Reset()
 			m.WriteHeader()
@@ -810,6 +821,8 @@ func TestAllocationsGetters(t *testing.T) {
 		new(ErrorCodeAttribute),
 	}
 	for i, g := range getters {
+		g := g
+		i := i
 		allocs := testing.AllocsPerRun(10, func() {
 			if err := g.GetFrom(m); err != nil {
 				t.Errorf("[%d] failed to get", i)
@@ -888,7 +901,7 @@ func BenchmarkMessage_CloneTo(b *testing.B) {
 	}
 	b.SetBytes(int64(len(m.Raw)))
 	a := new(Message)
-	m.CloneTo(a)
+	m.CloneTo(a) // nolint:errcheck,gosec
 	for i := 0; i < b.N; i++ {
 		if err := m.CloneTo(a); err != nil {
 			b.Fatal(err)
@@ -917,7 +930,7 @@ func TestMessage_AddTo(t *testing.T) {
 	if b.Equal(m) {
 		t.Fatal("should not be equal")
 	}
-	m.AddTo(b)
+	m.AddTo(b) // nolint:errcheck,gosec
 	if !b.Equal(m) {
 		t.Fatal("should be equal")
 	}
@@ -935,7 +948,7 @@ func BenchmarkMessage_AddTo(b *testing.B) {
 		b.Fatal(err)
 	}
 	a := new(Message)
-	m.CloneTo(a)
+	m.CloneTo(a) // nolint:errcheck,gosec
 	for i := 0; i < b.N; i++ {
 		if err := m.AddTo(a); err != nil {
 			b.Fatal(err)
@@ -945,7 +958,7 @@ func BenchmarkMessage_AddTo(b *testing.B) {
 
 func TestDecode(t *testing.T) {
 	t.Run("Nil", func(t *testing.T) {
-		if err := Decode(nil, nil); err != ErrDecodeToNil {
+		if err := Decode(nil, nil); !errors.Is(err, ErrDecodeToNil) {
 			t.Errorf("unexpected error: %v", err)
 		}
 	})
@@ -991,3 +1004,61 @@ func BenchmarkDecode(b *testing.B) {
 		}
 	}
 }
+
+func TestMessage_MarshalBinary(t *testing.T) {
+	m := MustBuild(
+		NewSoftware("software"),
+		&XORMappedAddress{
+			IP: net.IPv4(213, 1, 223, 5),
+		},
+	)
+	data, err := m.MarshalBinary()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Reset m.Raw to check retention.
+	for i := range m.Raw {
+		m.Raw[i] = 0
+	}
+	if err := m.UnmarshalBinary(data); err != nil {
+		t.Fatal(err)
+	}
+
+	// Reset data to check retention.
+	for i := range data {
+		data[i] = 0
+	}
+	if err := m.Decode(); err != nil {
+		t.Fatal(err)
+	}
+}
+
+func TestMessage_GobDecode(t *testing.T) {
+	m := MustBuild(
+		NewSoftware("software"),
+		&XORMappedAddress{
+			IP: net.IPv4(213, 1, 223, 5),
+		},
+	)
+	data, err := m.GobEncode()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Reset m.Raw to check retention.
+	for i := range m.Raw {
+		m.Raw[i] = 0
+	}
+	if err := m.GobDecode(data); err != nil {
+		t.Fatal(err)
+	}
+
+	// Reset data to check retention.
+	for i := range data {
+		data[i] = 0
+	}
+	if err := m.Decode(); err != nil {
+		t.Fatal(err)
+	}
+}
